<?php
namespace Teknisa\CADEST\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO; 
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\CADEST\Services;
use Zeedhi\Framework\Util\Functions;


class ParaVendasImposto extends CrudWithParameterBag {
    
    protected $dataSourceNames = array (
        'filterAliqImpFis'              => 'ger01900_cadastroproduto_venda_imposto',
        'filterImposto'                 => 'paraVendasImposto',
        'filterImpostoProduto'          => 'paraVendasImpostoProduto',
        'filterImpostoProdutos'         => 'paraVendasImpostoProdutos',
        'selectImpostoProduto'          => 'select_paraVendasImpostoProduto',
        'filterCliente'                 => 'paraVendasCliente',
        'filterBanco'                   => 'paraVendasBanco',
        'filterTabelaPreco'             => 'paraVendasClienteTabelaPreco',
        'filterTabelas'                 => 'paraVendasClienteTabelas',
        'selectTabPrec'                 => 'paraVendasClienteTabPrecSelect',
        'selectCst'                     => 'vnd10003_paramGerais_CST',
        'selectCstcsosn'                => 'vnd10003_paramGerais_CSTCSOSN',
        'selectCodcfop'                 => 'vnd10003_paramGerais_CODCFOP',
        'selectAliCONFINS'              => 'vnd10003_paramGerais_aliquotaCONFINS',
        'selectAliPIS'                  => 'vnd10003_paramGerais_aliquotaPIS',
        'selectCEST'                    => 'vnd10003_paramGerais_codCEST',
        'selectImpostoProduto_Exists'   => 'paraVendasImpostoProduto_Exists',
        'selectImpostoProdutos_Exists'  => 'paraVendasImpostoProdutos_Exists'
    );
    
    protected  $ParaVendasImposto;
    protected $crudService;
    
    protected  $parameterBagColumnsCFOP                   = array('NRORG'),
               $parameterBagColumnsCSTCSOSN               = array('NRORG'),
               $parameterBagColumnsImposto                = array('NRORG'),
               $parameterBagColumnsCST                    = array('NRORG'),
               $parameterBagColumnsCEST                   = array('NRORG'),
               $parameterBagColumnsClienteTabelaPreco     = array('NRORG'),
               $parameterBagColumnsBanco                  = array('NRORG'),
               $parameterBagColumnsCliente                = array('NRORG', 'CDFILIAL'),
               $parameterBagColumnsClienteTabelas         = array('NRORG', 'CDCLIENTE'),
               $parameterBagColumnsCONFINS_PIS            = array('NRORG', "CDSITUTRIBNFE"),
               $parameterBagColumnsSelectTabPrec          = array('NRORG', 'CDFILIAL', 'CDTABPREC'),
               $parameterBagColumnsImpostoProduto_Exists  = array('NRORG', 'CDFILIAL', 'CDPRODUTO'),
               $parameterBagColumnsImpostoProduto         = array('NRORG', 'CDFILIAL', 'TCDARVPROD', 'CDARVPROD','CDSEGMER','CDNVPRODUTO'), 
               $parameterBagColumnsSelectImpostoProduto   = array('NRORG', 'CDFILIAL', 'CDARVPROD' , 'LIMIT_CDARVPROD','TCDARVPROD'),
               $parameterBagColumnsImpostoProdutos        = array('NRORG', 'CDFILIAL', 'CDPRODINICIAL', 'CDPRODFINAL'), 
               $parameterBagColumnsImpostoProdutos_Exists = array('NRORG', 'CDFILIAL', 'CDPRODINICIAL', 'CDPRODFINAL'); 
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, $connection, Services\ParaVendasImposto $ParaVendasImposto, $crudService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->connection = $connection;
        $this->ParaVendasImposto = $ParaVendasImposto;
        $this->crudService = $crudService;
    }
    
    public function findImposto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterImposto'];
        $this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDPRODUTO');
        parent::find($request, $response);
    }
    
    public function deleteImpostoOdhen(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterAliqImpFis'];
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName($this->dataSourceName);
        $this->crudService->delete($response, $dataSet);
    }
    
    public function findAliqImpFis(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterAliqImpFis'];
        $this->parameterBagColumns = array('NRORG', 'CDSEGMER', 'CDPRODUTO');
        parent::find($request, $response);
    }
    
    public function saveImposto(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterImposto'];
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    } 
    
    public function findImpostoProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterImpostoProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsImpostoProduto;
         
        $conditions = $request->getFilterCriteria()->getConditions();
        $params = [];
        
        foreach ($conditions as $value) {
            $params[$value['columnName']] = $value['value'];
        }
        
        if(Functions::arrayKeyExists('CDARVPROD', $params)) {
            $params['CDARVPROD'] = str_replace("%", "", $params['CDARVPROD']) . '%';
        
            $filterCriteria = $this->createFilterCriteria($params);
            
            $newRequest = new DTO\Request\Filter (
                $filterCriteria,
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            ); 
        
            parent::find($newRequest, $response);
        } else {
            $params['TCDARVPROD'] = 'T';
            $params['CDARVPROD'] = null;
            $filterCriteria = $this->createFilterCriteria($params);
            
            $newRequest = new DTO\Request\Filter (
                $filterCriteria,
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            parent::find($newRequest, $response);
        }
    }
    
    
    private function createFilterCriteria($params) {
        $filterCriteria = new FilterCriteria ($this->dataSourceName);
        
        foreach($params as $name => $value) {
            $filterCriteria->addCondition($name, '=', $value);
        }
        
        return $filterCriteria;
    }
        
    public function saveImpostoProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        foreach($row['CDFILIAL'] AS $CDFILIAL){
            $params = array (
                'CDPRODUTO' => $row['CDPRODUTO'],
                'CDFILIAL'  => $CDFILIAL,
                'NRORG'     => $row['NRORG']
            );
            $filterCriteria = $this->createFilterCriteria($params);
            
            $newRequest = new DTO\Request\Filter (
                $filterCriteria,
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            ); 
            $checkImposto = $this->ParaVendasImposto->checkImposto($row, $CDFILIAL);
            $newResponse = new DTO\Response();
            
            $this->dataSourceName = $this->dataSourceNames['selectImpostoProduto_Exists'];
            $this->parameterBagColumns = $this->parameterBagColumnsImpostoProduto_Exists;
            parent::find($newRequest, $newResponse);
            
            $produto = $newResponse->getDataSets()[0]->getRows();
            if(!empty($produto)) {
                $this->ParaVendasImposto->deleteProdutoImposto($produto[0]);
            }
            // Insert or Update based on his searches results.
            $this->dataSourceName = $this->dataSourceNames['filterImpostoProduto'];
            $row['__is_new'] = (empty($produto));
            $row = $this->fixSaveImpostoProdutosParameters($row);
            $this->ParaVendasImposto->saveProdutoImposto($row, $CDFILIAL);
            $response->addDataSet(new DataSet($this->dataSourceName, array($row)));
            if($checkImposto['showMessage']){
                $response->addMessage(new Message( $checkImposto['message'], Message::TYPE_WARNING));
            } else {
                $response->addMessage(new Message('Certifique-se de que a aba Produto->Fiscal foi parametrizada. Unidades cadastradas, apenas alteradas!', Message::TYPE_MESSAGE));
            }
        }
    }
        
    public function saveExport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        foreach($row['CDFILIAL'] AS $CDFILIAL){
            $params = array (
                'CDPRODUTO' => $row['CDPRODUTO'],
                'CDFILIAL'  => $CDFILIAL,
                'NRORG'     => $row['NRORG']
            );
            $filterCriteria = $this->createFilterCriteria($params);
            $newRequest = new DTO\Request\Filter (
                $filterCriteria,
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            ); 
            $checkImposto = $this->ParaVendasImposto->checkImposto($row, $CDFILIAL);
            $newResponse = new DTO\Response();
            $this->dataSourceName = $this->dataSourceNames['selectImpostoProduto_Exists'];
            $this->parameterBagColumns = $this->parameterBagColumnsImpostoProduto_Exists;
            parent::find($newRequest, $newResponse);
            $produto = $newResponse->getDataSets()[0]->getRows();
            if(!empty($produto)) {
                $this->ParaVendasImposto->deleteProdutoImposto($produto[0]);
            }
            // Insert or Update based on his searches results.
            $this->dataSourceName = $this->dataSourceNames['filterImpostoProduto'];
            $row['__is_new'] = (empty($produto));
            $row = $this->fixSaveImpostoProdutosParameters($row);
            $this->ParaVendasImposto->saveProdutoImposto($row, $CDFILIAL);
            $response->addDataSet(new DataSet($this->dataSourceName, array($row)));
            if($checkImposto['showMessage']){
                $response->addMessage(new Message( $checkImposto['message'], Message::TYPE_WARNING));
            } else {
                $response->addMessage(new Message( 'Certifique-se de que a aba Produto->Fiscal foi parametrizada.', Message::TYPE_MESSAGE));
            }
        }
    }

    public function saveImpostoProdutos (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectImpostoProdutos_Exists'];
        $this->parameterBagColumns = $this->parameterBagColumnsImpostoProdutos_Exists;
        
        $row = $request->getDataSet()->getRows()[0];
        $params = array (
            'CDFILIAL'  => $row['CDFILIAL'],
            'NRORG'     => $row['NRORG']
        );
        
        $filterCriteria = $this->createFilterCriteria($params);
        
        $filterCriteria->addCondition( 'CDPRODINICIAL', FilterCriteria::GT , $row['CDPRODINICIAL']);
        $filterCriteria->addCondition( 'CDPRODFINAL', FilterCriteria::LTE, $row['CDPRODFINAL']  );
        
        $newRequest = new DTO\Request\Filter (
            $filterCriteria,
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        $newResponse = new DTO\Response();
        
        parent::find($newRequest, $newResponse);
        $productsAlreadyRegistered = $newResponse->getDataSets()[0]->getRows(); 
        
        $this->ParaVendasImposto->deleteProdutosImposto($productsAlreadyRegistered);
        
        
        $this->dataSourceName = $this->dataSourceNames['filterImpostoProdutos'];
        $this->parameterBagColumns = $this->parameterBagColumnsImpostoProdutos;
        
        $newResponse2 = new DTO\Response();
        
        parent::find($newRequest, $newResponse2);
        $productsToRegister = $newResponse2->getDataSets()[0]->getRows(); 
        
        $row = $this->fixSaveImpostoProdutosParameters($row);
        $this->ParaVendasImposto->saveProdutosImposto($productsToRegister, $row);
        
        $response->addDataSet(new DataSet($this->dataSourceName, $productsToRegister));
    }
    
    private function fixSaveImpostoProdutosParameters($row) {
        $row['VRPEALIMPFIS']    = Functions::arrayKeyExists('VRPEALIMPFIS', $row)    ? $row['VRPEALIMPFIS']      : null; 
        $row['VRPERCREDUCAO']   = Functions::arrayKeyExists('VRPERCREDUCAO', $row)   ? $row['VRPERCREDUCAO']     : null; 
        $row['IDMODALBASECALC'] = Functions::arrayKeyExists('IDMODALBASECALC', $row) ? $row['IDMODALBASECALC']   : null; 
        $row['IDINCIDEPISCOF']  = Functions::arrayKeyExists('IDINCIDEPISCOF', $row)  ? $row['IDINCIDEPISCOF']    : null; 
        $row['VRALIQIBPT']      = Functions::arrayKeyExists('VRALIQIBPT', $row)      ? $row['VRALIQIBPT']        : null; 
        $row['VRALIQIBPTES']    = Functions::arrayKeyExists('VRALIQIBPTES', $row)    ? $row['VRALIQIBPTES']      : null; 
        $row['CDCFOPPFIS']      = Functions::arrayKeyExists('CDCFOPPFIS', $row)      ? $row['CDCFOPPFIS']        : null; 
        $row['CDCSTICMS']       = Functions::arrayKeyExists('CDCSTICMS', $row)       ? $row['CDCSTICMS']         : null; 
        $row['CDCSTPISCOF']     = Functions::arrayKeyExists('CDCSTPISCOF', $row)     ? $row['CDCSTPISCOF']       : null; 
        $row['VRALIQPIS']       = Functions::arrayKeyExists('VRALIQPIS', $row)       ? $row['VRALIQPIS']         : null; 
        $row['VRALIQCOFINS']    = Functions::arrayKeyExists('VRALIQCOFINS', $row)    ? $row['VRALIQCOFINS']      : null;
        $row['CDCEST']          = Functions::arrayKeyExists('CDCEST', $row)          ? $row['CDCEST']            : null;  
        $row['__is_new'] = true;
        
        return $row;
    } 
    
    
    public function findBanco(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterBanco'];
        $this->parameterBagColumns = $this->parameterBagColumnsBanco;
        parent::find($request, $response);
    }
    
    public function saveBanco(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterBanco'];
        parent::save($request, $response);
    }
    
    public function findCliente(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterCliente'];
        $this->parameterBagColumns = $this->parameterBagColumnsCliente;
        
        $conditions = array();
        $filterCriteria = $request->getFilterCriteria();
        foreach($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if(in_array($columnName, $this->parameterBagColumnsCliente)) 
                $conditions[] = $condition;
        }
        
        $request = new DTO\Request\Filter(
            new FilterCriteria(
                $filterCriteria->getDataSourceName(),
                $conditions,
                $filterCriteria->getPage(),
                $filterCriteria->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        parent::find($request, $response);
    }
    
    // Trocar dataSourceName no request e usar o parent
    public function saveCliente(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterCliente'];
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    } 
    
    public function findTabelaPreco(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterTabelaPreco'];
        $this->parameterBagColumns = $this->parameterBagColumnsClienteTabelaPreco;
        parent::find($request, $response);
    }
    
    public function findTabelas(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['filterTabelas'];
        $this->parameterBagColumns = $this->parameterBagColumnsClienteTabelas;
        parent::find($request, $response);
    }
    
    // Essa Query poderia estar no query.php
    // Isso poderia estar em um service
    public function saveTabelas(DTO\Request\DataSet $request, DTO\Response $response) {
        $data = $request->getDataSet()->getRows();
        $queryLoja = "UPDATE CLIENFILIAL 
                    SET
                      CDTABEPREC = :CDTABEPREC,
                      CDCFILTABPRE = :CDCFILTABPRE
                    WHERE
                        (NRORG = :NRORG) 
                        AND 
                        (CDCLIENTE = :CDCLIENTE)
                        AND
                        (CDFILIAL = :CDFILIAL)";
                        
        $stmt = $this->connection->prepare($queryLoja); 
		$stmt->execute(array(
		    
		    "CDTABEPREC" => $data[0]['CDTABEPREC'],
		    "NRORG" => $data[0]['NRORG'],
		    "CDCLIENTE" => $data[0]['CDCLIENTE'],
		    "CDCFILTABPRE" => $data[0]['CDFILTABPREC'],
		    "CDFILIAL" => $data[0]['CDFILIAL']
            
		));
    }
    
    public function selectTabelaPreco(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectTabPrec'];
        $this->parameterBagColumns = $this->parameterBagColumnsSelectTabPrec;
        parent::find($request, $response);
    }
    
    public function selectCFOP(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectCodcfop'];
        $this->parameterBagColumns = $this->parameterBagColumnsCFOP;
        parent::find($request, $response);
    }
    
    public function selectCSTCSOSN(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectCstcsosn'];
        $this->parameterBagColumns = $this->parameterBagColumnsCSTCSOSN;
        $rows = $request->getFilterCriteria();
        $NRORG = $rows->getConditions()[0]['value'];
        $CDFILIALARRAY = $rows->getConditions()[1]['value'];
        
        //controle Regime Normal
        $controle = false;
        
        //arraySeaech
        $searchArray = array();
        
        //impostosCST Regime Normal
        $impostosCST = array();
        //impostosCSOSN Simples Nacional
        $impostosCSOSN = array(
                '0' => array(
                    'CDSITUTRIB' => '101', 
                    'DSSITUTRIB' => 'Tributada pelo Simples Nacional com permissão de crédito'
                ),
                '1' => array(
                    'CDSITUTRIB' => '102', 
                    'DSSITUTRIB' => 'Tributada pelo Simples Nacional sem permissão de crédito'
                ),
                '2' => array(
                    'CDSITUTRIB' => '103', 
                    'DSSITUTRIB' => 'Isenção do ICMS no Simples Nacional para faixa de receita bruta'
                ),
                '3' => array(
                    'CDSITUTRIB' => '201', 
                    'DSSITUTRIB' => 'Tributada pelo Simples Nacional com permissão de crédito e com cobrança do ICMS por substituição tributária'
                ),
                '4' => array(
                    'CDSITUTRIB' => '202', 
                    'DSSITUTRIB' => 'Tributada pelo Simples Nacional sem permissão de crédito e com cobrança do ICMS por substituição tributária'
                ),
                '5' => array(
                    'CDSITUTRIB' => '203', 
                    'DSSITUTRIB' => 'Isenção do ICMS no Simples Nacional para faixa de receita bruta e com cobrança do ICMS por substituição tributária'
                ),
                '6' => array(
                    'CDSITUTRIB' => '300', 
                    'DSSITUTRIB' => 'Imune'
                ),
                '7' => array(
                    'CDSITUTRIB' => '400', 
                    'DSSITUTRIB' => 'Não Tributada pelo Simples Nacional'
                ),
                '8' => array(
                    'CDSITUTRIB' => '500', 
                    'DSSITUTRIB' => 'ICMS cobrado anteriormente por substituição tributária (substituído) ou por antecipação'
                ),
                '9' => array(
                    'CDSITUTRIB' => '900', 
                    'DSSITUTRIB' => 'Outros'
                )
            );
        
        $impostosCSTCSOSN = array();
        if(!empty($CDFILIALARRAY)){
            foreach($CDFILIALARRAY as $value){
                $impostosCST = $this->ParaVendasImposto->getTipoImposto($value, $NRORG);
                $impostosCSTCSOSN = array_merge($impostosCSTCSOSN, $impostosCST);
                if(empty($impostosCST)){
                    $controle = true;
                }
            }
        }
        if($controle){
            $impostosCSTCSOSN = array_merge($impostosCSTCSOSN, $impostosCSOSN);
        }
        
        if(!empty($rows)){ 
            foreach($rows->getConditions() as $key => $value){
                if($value['operator'] == 'MAPPED_LIKE_ALL'){
                    $searchArray = $this->searchForArray($value['value'][0]['_ALL'], $impostosCSTCSOSN);
                }
            }
        }
        
        if(!empty($searchArray)){
            $impostosCSTCSOSN = $searchArray;
        }
        
        $response->addDataSet(new DataSet($this->dataSourceName, $impostosCSTCSOSN));
    }
    
    public function searchForArray($search, $array) {
       $retornoArray = array();
       $search = str_replace('%', '', $search);
       foreach ($array as $key => $val) {
           if ($val['CDSITUTRIB'] === $search) {
              $retornoArray[] = $val;
           }
           if (strstr(mb_strtoupper($val['DSSITUTRIB'], 'UTF-8'),  mb_strtoupper($search, 'UTF-8'))) {
               $retornoArray[] = $val;
           }
       }
       return $retornoArray;
    }
    
    
    
    public function selectCST(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectCst'];
        $this->parameterBagColumns = $this->parameterBagColumnsCST;
        parent::find($request, $response);
    }
    
    public function selectAliquotaCONFINS(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectAliCONFINS'];
        $this->parameterBagColumns = $this->parameterBagColumnsCONFINS_PIS;
        parent::find($request, $response);
    }
    
    public function selectAliquotaPIS(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectAliPIS'];
        $this->parameterBagColumns = $this->parameterBagColumnsCONFINS_PIS;
        parent::find($request, $response);
    }
    
    public function selectCodCEST(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectCEST'];
        $this->parameterBagColumns = $this->parameterBagColumnsCEST;
        parent::find($request, $response);
    }
    
    public function findSelectFieldImpostoProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNames['selectImpostoProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsSelectImpostoProduto;
        parent::find($request, $response);
    }
  
    public function generateAliquote (DTO\Request\Filter $request, DTO\Response $response) {
       try {
            $filter = $request->getFilterCriteria()->getConditions();
            $row["CDFILIAL"] = $filter[0]["value"];
            
            $bool = $this->ParaVendasImposto->generateAliquote($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function exportProduto (DTO\Request\Row $request, DTO\Response $response) {
       try {
            $row = $request->getRow();
            $i = 0;
            $row['IDUTILCTRLEST'] = 'N';
            foreach($row['CDPRODUTO'] AS $CDPRODUTO){
               
                if($row['IDUTILIMP'] == 'S'){
                    $this->ParaVendasImposto->saveIDUTILIMP($row, $CDPRODUTO);
                }
                if($row['IDUTILNCM'] == 'S'){
                    $this->ParaVendasImposto->saveIDUTILNCM($row, $CDPRODUTO);
                }
                if($row['IDUTILPREC'] == 'S'){
                    $this->ParaVendasImposto->saveIDUTILPREC($row, $CDPRODUTO);
                }
                if($row['IDUTILPARAMGER'] == 'S'){
                    $this->ParaVendasImposto->saveIDUTILPARAMGER($row,$CDPRODUTO);
                }
                if($row['IDUTILCTRLEST'] == 'S'){
                    $this->ParaVendasImposto->saveIDUTILCTRLEST($row, $CDPRODUTO);
                }
                foreach($row['CDLOJA'] AS $CDLOJA){
                    if($row['IDUTILPRODLOJA'] == 'S'){
                        $this->ParaVendasImposto->saveIDUTILPRODLOJA($row, $CDPRODUTO, $CDLOJA);
                    }
                    if($row['IDUTILLAYOUT'] == 'S'){
                        $this->ParaVendasImposto->saveIDUTILLAYOUT($row, $CDPRODUTO, $CDLOJA, $row['NMPRODUTO'][$i]);
                    }
                }
                $i++;
            }
            
            $response->addDataSet(new DataSet($this->dataSourceName, array($row)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}   