/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: 8120_CMPSTPO.sql

Descrio:
Calcula o limite de seguranca do produto considerando ou no os dias no teis.
Tambm retorna o nmero de dias de limite de segurana cadastrado para o produto.
Obs: Esta procedure foi baseada na 8100_CMPSTPO.sql;

Criao:
25/09/2004 Joo Fernando dos Reis Corra
*/

CREATE OR REPLACE
PROCEDURE LIMISEGU_CONS_DNUT(
   P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
   P_CDARVPROD    IN PRODUTO.CDARVPROD%TYPE,
   P_DTENTREGA    IN CALECOMP.DTENTREGA%TYPE,
   P_CDPERIENTR   IN PERIENTR.CDPERIENTR%TYPE,
   P_NRLIMISEGU   OUT NUMBER,
   P_NRLIMISEGCAD OUT LIMISEGPROD.NRLIMISEGU%TYPE) IS

   V_IDDNULIMSEG  VARCHAR2(1);
   V_NRLIMISEGU   NUMBER(5);
   V_NRCONTA      NUMBER(5);
   V_DTENTREGA    DATE;

   V_COUNT        FLOAT;

   /* Cursor com dias teis no intervalo de um ano a partir da data de entrega */
   CURSOR C_MAIORDATA IS
      (SELECT
          DTDATA
       FROM
          DATA
       WHERE
          (DTDATA BETWEEN P_DTENTREGA AND P_DTENTREGA + 364) AND
          ((V_IDDNULIMSEG = 'N') OR ((V_IDDNULIMSEG = 'S') AND
                                     (DTDATA NOT IN (SELECT
                                                        DTFERICOMP
                                                     FROM
                                                        FERICOMP
                                                     WHERE
                                                        (CDFILIAL   = P_CDFILIAL)  AND
                                                        (DTFERICOMP > P_DTENTREGA) AND
                                                         NOT EXISTS (SELECT 1
                                                                     FROM CALECOMP
                                                                     WHERE CALECOMP.CDFILIAL   = P_CDFILIAL
                                                                       AND CALECOMP.DTENTREGA  = DTFERICOMP
                                                                       AND CALECOMP.CDPERIENTR = P_CDPERIENTR) )))));

BEGIN
   V_NRLIMISEGU := 0;
   V_DTENTREGA  := P_DTENTREGA;

   SELECT
      NVL(MAX(TO_NUMBER(NRLIMISEGU)),0),
      MAX(NRLIMISEGU)
   INTO
      V_NRLIMISEGU,
      P_NRLIMISEGCAD
   FROM
      LIMISEGPROD,
      PRODUTO
   WHERE
      (LIMISEGPROD.CDPRODUTO = PRODUTO.CDPRODUTO) AND
      (LIMISEGPROD.CDFILIAL  = P_CDFILIAL)        AND
      (PRODUTO.CDARVPROD     = (SELECT /*+ INDEX (A PK_LIMISEGPROD) INDEX (B PK_PRODUTO) */
                                   MAX(CDARVPROD)
                                FROM
                                   LIMISEGPROD A,
                                   PRODUTO B
                                WHERE
                                   (A.CDPRODUTO = B.CDPRODUTO) AND
                                   (A.CDFILIAL  = P_CDFILIAL)  AND
                                   (INSTR(P_CDARVPROD,B.CDARVPROD) = 1)));

   IF V_NRLIMISEGU > 0 THEN
      /* Busca o parmetro que indica se considera dias no teis */
      SELECT
         MAX(PERIENTR.IDDNULIMSEGP) INTO V_IDDNULIMSEG
      FROM
         PERIENTR
      WHERE
         (PERIENTR.CDPERIENTR = P_CDPERIENTR);
      IF V_IDDNULIMSEG IS NULL THEN
         SELECT
            NVL(MAX(PARASUPRI.IDDNULIMSEG), 'N') INTO V_IDDNULIMSEG
         FROM
            PARASUPRI,
            FILIAL
         WHERE
            (FILIAL.CDFILIAL    = P_CDFILIAL) AND
            (PARASUPRI.CDFILIAL = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL));
      END IF;

      OPEN C_MAIORDATA;
      FOR V_NRCONTA IN 0..V_NRLIMISEGU LOOP
         /* Busca a ltima data ate o limite de segurana */
         FETCH C_MAIORDATA INTO V_DTENTREGA;
         EXIT WHEN C_MAIORDATA%NOTFOUND;
      END LOOP;
      CLOSE C_MAIORDATA;
      IF V_IDDNULIMSEG = 'S' THEN
         LOOP /* loop para adiantar a data de entrega caso o dia anterior a entrega seja nao util */
            SELECT
               COUNT(*) INTO V_COUNT
            FROM
               FERICOMP
            WHERE
               (CDFILIAL   = P_CDFILIAL)     AND
               (DTFERICOMP = V_DTENTREGA - 1)AND
                NOT EXISTS (SELECT 1
                            FROM CALECOMP
                            WHERE CALECOMP.CDFILIAL   = P_CDFILIAL
                              AND CALECOMP.DTENTREGA  = DTFERICOMP
                              AND CALECOMP.CDPERIENTR = P_CDPERIENTR);
            IF V_COUNT > 0 THEN
               V_DTENTREGA := V_DTENTREGA - 1;
            ELSE
               EXIT;
            END IF;
         END LOOP;
      END IF;

      V_NRLIMISEGU := V_DTENTREGA - P_DTENTREGA;
   END IF;
   P_NRLIMISEGU := V_NRLIMISEGU;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('LIMISEGU_CONS_DNUT','5.07.002'); END;
/

